import json
import requests
from datetime import datetime, date
import time
from dateutil.relativedelta import relativedelta
import logging
import pandas as pd
import sys

today = date.today()
today = today.strftime("%d_%m_%Y")

logging.basicConfig(filename=("logs/aw_orders_" + today +".log"), level=logging.INFO, format='%(asctime)s %(levelname)s:%(message)s', datefmt='%Y-%m-%d %H:%M:%S')

ONLY_AW_FLAG = False
AW_URL = 'http://130.180.194.2:3004'
GS_TOKEN = "3a-0y\pzFWg6?4O2j5DAYwn=v8`rCBd(mScuGiKJ.)^LNQPVx1EsHkRITUl9X>7M"
SH_TOKEN = "+MCJLP-14Qp.q0X$WRztG{|SUA3bHuFlvo8sny@VOaE^7N29mdYk6]c*DhjwI5ZT"

def get_order_from_sellasist(order_id):
    print('get_order_from_sellasist')
    logging.info('get_order_from_sellasist')
    '''
    Orders are passed further without any edition
    '''
    headers = {'apiKey': 'nng8l267hdzJaDE6k7WZSTgcULCU0KtW'}
    
    r = requests.get(f"https://shumee.sellasist.pl/api/v1/orders/{order_id}", headers=headers)
    
    logging.info(r.status_code)
    if 'application/json' in r.headers['Content-Type']:
        logging.info(r.json())
        return r.json()
    else:
        logging.info(r.text)
        return json.loads(r.text)
    
def get_orders_from_sellasist(status):
    print('get_order_from_sellasist')
    logging.info('get_order_from_sellasist')
    '''
    Orders are passed further without any edition
    '''
    four_hrs_before = datetime.now() - relativedelta(days=3)
    four_hrs_before = four_hrs_before.strftime("%Y-%m-%d %H:%M:%S")
    params = {
        "status_id": status,
        "date_from": four_hrs_before,
        }
    
    headers = {'apiKey': 'nng8l267hdzJaDE6k7WZSTgcULCU0KtW'}
    
    r = requests.get("https://shumee.sellasist.pl/api/v1/orders_with_carts", headers=headers, params=params)
    
    logging.info(r.status_code)
    if 'application/json' in r.headers['Content-Type']:
        logging.info(r.json())
        return r.json()
    else:
        logging.info(r.text)
        return json.loads(r.text)

def get_orders_from_csv():
    orders = pd.read_csv('orders.csv', delimiter = ';', keep_default_na=False).to_dict('records')
    return orders

def check_duplicates(orders):
    saved_orders = get_orders_from_csv()
    saved_orders_ids = []
    orders_to_send_to_aw = []
    
    for x in saved_orders:
        saved_orders_ids.append(int(x['sellasist_order_id']))
    
    for z in orders:      
        print(json.dumps(z, indent=4))  
        if int(z['id']) not in saved_orders_ids:
            for prod in z['carts']:
                if 'aw_' in prod['name']:
                    orders_to_send_to_aw.append(z)
                    break
    return orders_to_send_to_aw

def save_orders_to_csv(orders, mode):
    df = []
    if mode == 'a':
        for order in orders:
            df.append(
                {
                    'sellasist_order_id': order['id'],
                    'aw_id': order['aw_id'],
                    'tracking': '',
                    'company': order['our_company'],
                    'aw_inner_id': ''
                }
            )
        df = pd.DataFrame.from_dict(df)
        df.to_csv('orders.csv', sep=';', mode = mode, index=False, header=False)
    elif mode == 'w':
        for order in orders:
            df.append(
                {
                    'sellasist_order_id': order['sellasist_order_id'],
                    'aw_id': order['aw_id'],
                    'tracking': order['tracking'],
                    'company': order['company'],
                    'aw_inner_id': ''
                }
            )
        df = pd.DataFrame.from_dict(df)
        df.to_csv('orders.csv', sep=';', mode = mode, index=False, header=True)
    
def send_order_to_aw(order, token):
    print('send order to aw')
    print(order)
    orderedProducts = []
    aw_only_flag = True
    cod = False

    for x in order['carts']:
        if 'aw_' in x['name']:
            if ("MALL" in x['name'] or "MIMOVRSTE" in x['name']) and (float(order['payment']['paid']) == float(0)):
                cod = True
            orderedProducts.append(
                {
                    "productId": int(x['name'].replace(' aw_', '').split(":")[1]),
                    "quantity": x['quantity']
                }
            )
        else:
            aw_only_flag = False

    headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json',
        'Authorization': 'Bearer ' + token,
    }

    address = order['bill_address']

    data = {
        "orderNumber": str(order['id']),
        "deliveryPoint": {
            "name": f"{address['name']} {address['surname']}",
            "street": address['street'],
            "city": address['city'],
            "postalCode": address['postcode'],
            "country": address['country']['code'],
            "phone": address['phone']
        },
        "payment": {
            "method": "BANK_TRANSFER",
            "amount": float(order['payment']['paid'])
        },
        "orderedProducts": orderedProducts
    }
    if cod:
        data["deliveryPoint"] = {
            "name": "Olzalogistic Shumee COD",
            "street": "ul. Cieszyńska 11",
            "city": "Hażlach",
            "postalCode": "43-419",
            "country": "PL",
            "phone": "+420735909436"
        }
    #print("would be sent to aw ")
    print(data)
    r = requests.post(AW_URL+'/api/dropshipping/orders', data=json.dumps(data), headers=headers)
    print(r.text)
    try:
        return json.loads(r.text)['orderId'], aw_only_flag
    except:
        return r.text, aw_only_flag
    
def get_inner_aw_id(order):
    print('get inner aw id')
    logging.info('get inner aw id')
    try:
        order_number = int(order['aw_id'])
    except:
        order_number = -1
    if order['company'] == 'greatstore':
        headers = {
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + GS_TOKEN,
        }
    elif order['company'] == 'shumee':
        headers = {
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + SH_TOKEN,
        }

    r = requests.get(f'http://130.180.194.2:3004/api/dropshipping/orders/{order_number}/status', headers=headers)
    print(r.text)
    logging.info(r.text)
    try:
        return json.loads(r.text)['innerOrderNumber']
    except:
        return ''

def send_orders_to_aw(orders):
    print('send orders to aw')

    for x in orders:
        if (int(x['status']['id']) == 7) or (int(x['status']['id']) == 15):
            x['aw_id'], x['only_aw'] = send_order_to_aw(x, GS_TOKEN)
            x['our_company'] = 'greatstore'
        elif (int(x['status']['id']) == 21) or (int(x['status']['id']) == 28):
            x['aw_id'], x['only_aw'] = send_order_to_aw(x, SH_TOKEN)
            x['our_company'] = 'shumee'
        x['inner_aw_id'] = ''
    return orders
    
def update_sellasist_order(orders):
    print('edit_order')
    for order in orders:
        if order['inner_aw_id'] != '':
            aw_id = order['inner_aw_id']
        else:
            aw_id = order['aw_id']
        try:
            sellasist_order_id = order['id']
        except:
            sellasist_order_id = order['sellasist_order_id']
        params = {
            "additional_fields": [
                {
                    "field_id": 1,
                    "field_value": aw_id
                }
            ]
        }
    
        headers = {'apiKey': 'nng8l267hdzJaDE6k7WZSTgcULCU0KtW'}
        
        r = requests.put(f"https://shumee.sellasist.pl/api/v1/orders/{sellasist_order_id}", headers=headers, params=params)
        
        logging.info(r.status_code)
        if 'application/json' in r.headers['Content-Type']:
            logging.info(r.json())
            return r.json()
        else:
            logging.info(r.text)
            return json.loads(r.text)
        
def set_sellasist_status(orders):
    print('edit_order')
    for order in orders:
        #TODO na jakie statusy ma się zmieniać
        if order['only_aw'] == True:
            params = {
                "status_id": '',
            }
            if int(order['status']) == 28:
                #shumee
                params['status'] = 30
            elif int(order['status']) == 7:
                #greatstore
                params['status'] = 43
            elif int(order['status']) == 15:
                #greatstore
                params['status'] = 51
            elif int(order['status']) == 21:
                #shumee
                params['status'] = 23
            
        headers = {'apiKey': 'nng8l267hdzJaDE6k7WZSTgcULCU0KtW'}
        
        r = requests.put(f"https://shumee.sellasist.pl/api/v1/orders/{order['id']}", headers=headers, params=params)
        
        logging.info(r.status_code)
        if 'application/json' in r.headers['Content-Type']:
            logging.info(r.json())
            return r.json()
        else:
            logging.info(r.text)
            return json.loads(r.text) 
        
def check_status(order):
    try:
        order_number = int(order['aw_id'])
    except:
        order_number = -1
    if order['company'] == 'greatstore':
        headers = {
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + GS_TOKEN,
        }
    elif order['company'] == 'shumee':
        headers = {
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + SH_TOKEN,
        }

    r = requests.get(f'http://130.180.194.2:3004/api/dropshipping/orders/{order_number}/status', headers=headers)
    print(r.text)
    try:
        return json.loads(r.text)['shipping']['trackingNumber']
    except:
        return ''
    
def upload_tracking_to_sellasist(order):
    print('add_tracking_to_sellasist')
    
    headers = {'apiKey': 'nng8l267hdzJaDE6k7WZSTgcULCU0KtW'}
    params = {
        "date": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        "service": "DPD",
        "service_internal": "DPD",
        "order_id": order['sellasist_order_id'],
        "status": "Wygenerowano przesyłkę B2B",
        "tracking_number": order['tracking']
    }
    r = requests.post(f'https://shumee.sellasist.pl/api/v1/ordersshipments', headers=headers, params = params)
    logging.info(r.status_code)
    if 'application/json' in r.headers['Content-Type']:
        logging.info(r.json())
        return r.json()
    else:
        logging.info(r.text)
        return json.loads(r.text)
    
def check_statuses():
    print('check statuses')
    orders = get_orders_from_csv()
    for x in orders:
        if x['tracking'] == '':
            status = check_status(x)
            if status != '':
                x['tracking'] = status
                upload_tracking_to_sellasist(x)
            if x['inner_aw_id'] == '':
                status = get_inner_aw_id(x)
                if status != '':
                    x['inner_aw_id'] = status
                    if status != '':
                        update_sellasist_order(x)
    return orders

def main():
    orders = []
    #TODO statusy do zmiany
    if len(sys.argv) > 1:
        for order_id in sys.argv[1].split(","):
            orders.append(get_order_from_sellasist(order_id))
    else:
        for x in [7,15,21,28]:
            orders += get_orders_from_sellasist(x)

    orders = check_duplicates(orders)  
    orders = send_orders_to_aw(orders)
    update_sellasist_order(orders)
    set_sellasist_status(orders)
    save_orders_to_csv(orders, 'a')
    
    orders = check_statuses()

    save_orders_to_csv(orders, 'w')
    
    
if __name__ == "__main__":
    start_time = time.time()
    main()
    end_time = time.time()
    with open("/usr/home/shumeeshumee/domains/shumeeshumee.usermd.net/public_html/times.csv", 'a') as f:
        f.write(datetime.now().strftime("%d-%m-%y_%H:%M")+";aw_orders_sellasist;"+str(round(end_time-start_time, 2))+"\n")





